@@ -1,5 +1,5 @@
 /*
- * Copyright  2000 SuSE, Inc.
+ * Copyright © 2000 SuSE, Inc.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -23,11 +23,6 @@
 
 #include "icint.h"
 
-#ifndef __GNUC__
-#define __inline
-#endif
-
-
 #define cvt8888to0565(s)    ((((s) >> 3) & 0x001f) | \
 			     (((s) >> 5) & 0x07e0) | \
 			     (((s) >> 8) & 0xf800))
@@ -857,8 +852,6 @@ pixman_compositeSolidMask_nx1xn (pixman_operator_t   op,
 	      0x0);
 }
 
-# define mod(a,b)	((b) == 1 ? 0 : (a) >= 0 ? (a) % (b) : (b) - (-a) % (b))
-
 void
 pixman_composite (pixman_operator_t	op,
 	     pixman_image_t	*iSrc,
@@ -884,6 +877,12 @@ pixman_composite (pixman_operator_t	op,
     int	    dstAlphaMap = iDst->alphaMap != 0;
     int		    x_msk, y_msk, x_src, y_src, x_dst, y_dst;
     int		    w, h, w_this, h_this;
+
+    if (iSrc->pixels->width == 0 ||
+	iSrc->pixels->height == 0)
+    {
+	return;
+    }
     
     xDst += iDst->pixels->x;
     yDst += iDst->pixels->y;
@@ -993,7 +992,6 @@ pixman_composite (pixman_operator_t	op,
 	{
 	    switch (iSrc->format_code) {
 	    case PICT_a8r8g8b8:
-	    case PICT_x8r8g8b8:
 		switch (iDst->format_code) {
 		case PICT_a8r8g8b8:
 		case PICT_x8r8g8b8:
@@ -1008,7 +1006,6 @@ pixman_composite (pixman_operator_t	op,
 		}
 		break;
 	    case PICT_a8b8g8r8:
-	    case PICT_x8b8g8r8:
 		switch (iDst->format_code) {
 		case PICT_a8b8g8r8:
 		case PICT_x8b8g8r8:
@@ -1078,6 +1075,14 @@ pixman_composite (pixman_operator_t	op,
 	func = pixman_compositeGeneral;
 	break;
     }
+    /* if we are transforming, we handle repeats in
+     * IcFetch[a]_transform
+     */
+    if (iSrc->transform)
+      srcRepeat = 0;
+    if (iMask && iMask->transform)
+      maskRepeat = 0;
+    
     n = pixman_region_num_rects (region);
     pbox = pixman_region_rects (region);
     while (n--)
@@ -1095,13 +1100,13 @@ pixman_composite (pixman_operator_t	op,
 	    x_dst = pbox->x1;
 	    if (maskRepeat)
 	    {
-		y_msk = mod (y_msk, iMask->pixels->height);
+		y_msk = MOD (y_msk, iMask->pixels->height);
 		if (h_this > iMask->pixels->height - y_msk)
 		    h_this = iMask->pixels->height - y_msk;
 	    }
 	    if (srcRepeat)
 	    {
-		y_src = mod (y_src, iSrc->pixels->height);
+		y_src = MOD (y_src, iSrc->pixels->height);
 		if (h_this > iSrc->pixels->height - y_src)
 		    h_this = iSrc->pixels->height - y_src;
 	    }
@@ -1110,13 +1115,13 @@ pixman_composite (pixman_operator_t	op,
 		w_this = w;
 		if (maskRepeat)
 		{
-		    x_msk = mod (x_msk, iMask->pixels->width);
+		    x_msk = MOD (x_msk, iMask->pixels->width);
 		    if (w_this > iMask->pixels->width - x_msk)
 			w_this = iMask->pixels->width - x_msk;
 		}
 		if (srcRepeat)
 		{
-		    x_src = mod (x_src, iSrc->pixels->width);
+		    x_src = MOD (x_src, iSrc->pixels->width);
 		    if (w_this > iSrc->pixels->width - x_src)
 			w_this = iSrc->pixels->width - x_src;
 		}
